const config = require('../config.json');
const baseFcn = require('./baseFcn.js');

const fs = require("fs")


var self = {
  // 備份預設目錄
  defaultBackupPath: function () {
    let userName = require("os").userInfo().username
    return `C:/Users/${userName}/Desktop/couchDB_Data_Backup`
  },

  // 取得最新的檔案集
  getLatestBackupFiles: function (path) {
    let files = self.readBackupFiles(path)
    let assortedFiles = self.assortBackupFiles(files)

    let latestFiles = Object.keys(assortedFiles).map((dbName) => {
      return assortedFiles[dbName][0]
    })

    return latestFiles
  },

  // 依據 dbName 分類檔案，以 timestamp 大至小
  assortBackupFiles: function (files) {
    let assortedFiles = {
      /*dbName: [
        {
          "fileName": "ttmatch_datas-2020-02-26-5E554F0B.json",
          "dbName": "ttmatch_datas",
          "timestamp": 1582649099
        },
      ]//*/
    }

    files.forEach((file) => {
      let dbName = file.dbName;

      // 檢查是否已有此 dbName 類別
      if (assortedFiles[dbName]) {
        // 加入 file
        assortedFiles[dbName].push(file)
        return
      }

      // 沒有此類別則生成矩陣
      assortedFiles[dbName] = [];
      assortedFiles[dbName].push(file)
    })

    // 每個類別依照 timestamp 排序
    Object.keys(assortedFiles).forEach((dbName) => {
      assortedFiles[dbName].sort((a, b) => {
        return a.timestamp > b.timestamp ? -1 : 1
      })
    })

    return assortedFiles
  },
  // 讀取備份檔案集
  readBackupFiles: function (path) {
    let allFiles = fs.readdirSync(path)

    // 解析檔案
    let detailFiles = allFiles.map((file) => {
      let fileName = file.split(".")[0]
      let splitName = fileName.split("-")

      let dbName = splitName[0]
      let timestamp = parseInt(splitName[splitName.length - 1], 16)

      return {
        fileName: file, dbName, timestamp
      }
    })

    return detailFiles
  },

  // 是否為有效路徑
  isValidPath: function (path) {
    try {
      fs.statSync(path);
      return true
    } catch (error) {
      return false
    }
  },

  // 建立路徑
  mkdir: function (dir) {
    fs.mkdirSync(dir, err => {
      if (err) return false;
      return true;
    })
  },

  // 根據 dbName 生成檔名
  fileName: function (dbName) {
    return `${dbName}-${self.fileTimestamp()}.json`
  },
  // 檔名時間戳記
  fileTimestamp: function () {
    let timestamp = baseFcn.getUnix().toString(16).toUpperCase();
    return `${baseFcn.getNow("YYYYMMDD")}-${timestamp}`
  },

  // 寫入 JSON 檔
  writeJson: function (path = "", name = "", json = [], step = "") {
    if (name === "")
      name = `json_file_${baseFcn.getNow()}.json`

    if (path === "")
      path = self.defaultBackupPath()

    path += ("/" + name);

    let data = JSON.stringify(json, null, 2)

    return new Promise((resolve, reject) => {
      fs.writeFile(path, data, function (err) {
        if (err)
          return reject({
            step, status: 500,
            msg: "寫入 JSON 發生錯誤", reason: err
          });

        resolve(name);
      });
    })

  },

  // 讀取 JSON 檔
  readJson: function (path = "", name = "", step = "") {
    return new Promise((resolve, reject) => {
      fs.readFile(`${path}/${name}`, (err, data) => {
        if (err)
          return reject({
            step, status: 500,
            msg: "讀取 JSON 發生錯誤", reason: err
          });

        let json = JSON.parse(data)

        let info = fs.statSync(`${path}/${name}`);

        resolve({
          data: json,
          size: info.size
        })
      })
    })
  },
}

module.exports = self